/****************************************************************************
* Modulo : gbart.c
* Fecha  : 19/09/1999
* Autor  : Federico Bareilles
* Version: 0.1-0
* Descripcion:
*
* 
*****************************************************************************/
#define VERSION "0.1-41"

#define _POSIX_SOURCE 1		/* POSIX compliant source  */

#include <stdio.h>
#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <getopt.h>
#include <string.h>
//#include <gtk/gtk.h>

#include "gbart_gtk.h"
#include "gbart.h"

extern char **environ;

/* Flags: */
int verbose_flag      = 0;
int primaria_activa_flag = 1;
int secundaria_activa_flag = 1;
int periodo_variable_flag = 1;
int excentricidad_variable_flag = 1;
int sterne_flag = 1;


char *ProgramName;
int Salir = 0;

/* fifo de comunicacion con el GnuPlot: */
FILE *fp_gpc; /* gpc = GnuPlot Command */



char *HelpText[] ={
  "  -v\t\t\t- verbose mode.",
  "  --help\t\t- this help !!!.",
  ""
};


char *gnuplot_set_x11[] = {
  "set term x11",
  "set output", /* default output */
  /*"set grid",*/
  /*  "set mxtics 4",*/
  "set nokey",
  "set nozeroaxis",
  /*"set title 'Ejemplo'",*/
  "set ylabel 'V.R.'",
  "set xlabel 'Fase'",
  ""
};

char *gnuplot_command[] = {
  "gnuplot",
  "-geometry 640x480-4+0",
  /*"-xrm \"gnuplot*background: #d0d4d0\"",*/ /* Ojo en 8 bbits, 
					     tendria que ser #d7d7d7 */
  /*
  "-xrm \"gnuplot*line1Color: maroon\"",
  "-xrm \"gnuplot*line2Color: navyblue\"",
  "-xrm \"gnuplot*line3Color: gray20\"",
  "-xrm \"gnuplot*line4Color: gray40\"",
  */
  "-pointsize 2",
  ""
};


/* make a clean exit on interrupts */
void cleanup(int s)
{ 
  if( verbose_flag )
    if( !Salir )
      fprintf(stderr, "\nSending signal %d to %s.\n", s,ProgramName);
  Salir = 1;
}


void cartel()
{
  fprintf(stderr, 
	  "GTK bart, Universidad Nacional de La Plata v%s\n", 
	  VERSION );
  fprintf(stderr,
	  "Federico Bareilles. email: fede@lahuan.fcaglp.unlp.edu.ar\n");
}


void usage()
{
  cartel();
  fprintf(stderr,
	  "Usage: \t%s [-v][--help]\n", 
	  ProgramName);
}


void Help()
{
  int i=0;

  while((char) *HelpText[i] != 0)
    fprintf(stderr,"%s\n",HelpText[i++]);
 fprintf(stderr,"\n");
}


#define DATA_SERIE_MAX 0

int main(int argc, char **argv)
{
  char FIFO_gpc[_NAME_MAX];
  char data_serie[DATA_SERIE_MAX][_NAME_MAX];
  char *cp;
  char command[512];
  char *display;
  pid_t gnuplot_pid;
  int status, i;
  GtkStyle *st;
  /*
    struct sigaction sa;
  */
  int op;
  static struct option  const  longopts[] =
  {
    {"verbose", 0, 0, 'v'},
    {"help", 0, 0, 101},
    {"display", 1, 0, 102},
    {0,  0,  0,  0}
  };

  if ((cp = strrchr(argv[0], '/')) != NULL)
    ProgramName  = cp + 1;
  else
    ProgramName  = argv[0];
  /* sprintf(display,":0"); */
  display = getenv("DISPLAY"); 
  //  gtk_init (&argc, &argv);

  opterr =  0;
  while  ((op = getopt_long(argc, argv, "v",longopts, 0)) != EOF)
    switch( op ){
    case 'v':
      verbose_flag = 1;
      break;
    case 102:
      display = optarg;
      break;
    case 101:
      usage();
      Help();
      exit(-1);
    default:
      usage();
      exit(-1);
    };
   cartel();
   
   gtk_init (&argc, &argv);

  /* Obtngo un nombre para la fifo de comunicacion con el gnuplot */
  if(tmpnam( FIFO_gpc ) == NULL){
    perror("tmpnam->FIFO_gpc");
    exit(1);
  }
  if( verbose_flag )
    fprintf(stderr,"FIFO_gpc %s\n", FIFO_gpc);

  for(i=0;i<DATA_SERIE_MAX;i++){ /* Archivos Temporarios */ 
    if(tmpnam( data_serie[i] ) == NULL){
      perror("tmpnam->data_serie");
      exit(1);
    }
    if( verbose_flag )
      fprintf(stderr,"serie %d  %s\n", i+1, data_serie[i]);
  }

  umask(0);
  /* creo la fifo */
  if( mkfifo(FIFO_gpc, 0600) == -1){
    perror("Erro al crear la FIFO");
    exit(1);
  }

  /* Genero la interface GTK: */
  my_gtk_create();
  st = gtk_widget_get_default_style();
  
  /* lanzo gnuplot */
  /*sprintf(command,"gnuplot -geometry 640x480-0+0  < %s",FIFO_gpc);*/
  *(command) = 0;
  for(i=0; (char) *gnuplot_command[i] != 0; i++)
    sprintf(command+strlen(command),"%s ", gnuplot_command[i]);
  sprintf(command+strlen(command),
	  "-xrm \"gnuplot*background: #%02x%02x%02x\" ", 
	  st->bg[0].red, st->bg[0].green, st->bg[0].blue);
  sprintf(command+strlen(command),"-title \"%s: plot\" ", ProgramName);
  sprintf(command+strlen(command),"-display %s ", display);
  sprintf(command+strlen(command),"< %s",FIFO_gpc);
  if( verbose_flag )
    fprintf(stderr,"gnuplot command:\n%s\n", command);
  gnuplot_pid = my_exec( command );
  if( verbose_flag )
    fprintf(stderr,"pid: %d\n", gnuplot_pid);

  /* habro la fifo de comunicacion con el guplot */
  if((fp_gpc = fopen(FIFO_gpc, "w")) == NULL) {
    perror("fopen");
    exit(1);
  }

  /* seteo el gnuplot */
  for(i=0; (char) *gnuplot_set_x11[i] != 0; i++)
    fprintf(fp_gpc,"%s\n", gnuplot_set_x11[i]);

  /* mando un garabato */
  /*
  for(i=10;i<50;i++){
    fprintf(fp_gpc,"set xrange [-%d:%d]\n",i,i);
    fprintf(fp_gpc,"plot cos(x)\n");
    fflush(fp_gpc);
    //usleep(200000);
  }
  */

  gtk_main();

  fprintf(fp_gpc,"quit\n");
  fflush(fp_gpc);

  /* cierro la fifo de comunicacion con el gnuplot y espero que este termine */
  fclose(fp_gpc);
  waitpid(gnuplot_pid, &status, 0);/* Espero que termine gnuplot */
  if( status )
    fprintf(stderr,"Proceso %d termina por la se~nal %d.\n", 
	    gnuplot_pid, status);
  /* elimino la fifo de comunicacion */
  unlink(FIFO_gpc);

  if( verbose_flag )
    fprintf(stderr,"Exit...\n");

  return 0;
}


pid_t my_exec(char *command)
{
  pid_t pid;

  if (command == 0)
    return 1;
  pid = fork();
  if (pid == -1)
    return -1;
  if (pid == 0) {
    char *argv[4];
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    if( execve("/bin/sh", argv, environ) == -1){
      perror("execve");
      exit(1); /* ojo: execve no retorna si tiene exito. */
    }
    exit(127);
  }
  return pid;
}

int my_system(char *command) 
{
  int status;
  pid_t pid;

  if (command == 0)
    return 1;
  pid = fork();
  if (pid == -1)
    return -1;
  if (pid == 0) {
    char *argv[4];
    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = command;
    argv[3] = 0;
    execve("/bin/sh", argv, environ);
    exit(127);
  }
  /*
  fprintf(stderr,"Comando %s, pid %d\n", command, pid);
  */
  print_mess("PID: %d", pid);
  fprintf(stderr,"PID: %d\n", pid);
  do {
    if (waitpid(pid, &status, 0) == -1) {
      if (errno != EINTR)
	return -1;
    } 
    else
      return status;
  } while(1);
}


